module App

    class Stacks

        SCHEMA_FILE              = "#{App::Opt::get_base_path}#{App::Opt::OPT_PATH}/awx/stacks-schema.yml"

        @@stacks = nil

        # Takes a Hash that will have all the deployment info.
        # @return void
        def self.init(stacks)
            raise RuntimeError, 'Cannot run App::Stacks::init() more than once.' unless @@stacks.nil?
            raise RuntimeError, "Need either a Local or S3Bucket key, found neither: #{stacks.keys}" unless stacks.has_key?('Local') || stacks.has_key?('S3Bucket')
            @@stacks = {}

            if stacks.has_key?('Local')
                source_file = File.expand_path(stacks['Local']['File'])
                # Throw error if source file doesn't exist.
                Blufin::Terminal::error("Cannot find source file: #{Blufin::Terminal::format_directory(source_file)}") unless Blufin::Files::file_exists(source_file)
                # Validate the source file against the expected schema.
                process_source_file(source_file)
            elsif stacks.has_key?('S3Bucket')

                # TODO - Finish this once we start running this on an EC2 instance (build/deploy server).
                # TODO - Whatever file we validate should be available on disk locally.
                # TODO - If the source is an S3 bucket, pull it down into a /tmp folder (on EC2 instance) and validate from there.

            end

        end

        private

        # Validate the Deployment YML.
        # @return void
        def self.process_source_file(source_file)
            # Skip empty file.
            return if Blufin::Files::is_empty(source_file)
            # Otherwise, validate file.
            document, errors = Blufin::Config::validate_file(source_file, SCHEMA_FILE)
            if errors && !errors.empty?
                errors_output = []
                errors.each { |e| errors_output << "[#{e.path}] #{e.message}" }
                Blufin::Terminal::error("Your configuration file is invalid. Please fix: #{Blufin::Terminal::format_directory(source_file)}", errors)
            else
                begin
                    file_parsed = YAML.load_file(source_file)
                rescue => e
                    Blufin::Terminal::error("Failed to parse config file: #{Blufin::Terminal::format_directory(source_file)}", e.message)
                end

                # TODO NOW - FINISH THIS...
                puts file_parsed.to_yaml
                exit

            end

        end

    end

end