module AppCommand

    class DynamoDB < ::Convoy::ActionCommand::Base

        JAR_FILE = 'DynamoDBLocal.jar'

        def execute

            begin

                @opts = command_options
                @args = arguments

                @dynamo_db_path = Blufin::Config::get['DynamoDBPath']
                @dynamo_db_file = "#{@dynamo_db_path}/#{JAR_FILE}"

                opts_validate
                opts_routing

            rescue => e

                Blufin::Terminal::print_exception(e)

            end

        end

        def opts_validate

            # Make sure DynamoDB .jar file exists.
            Blufin::Terminal::error("File not found: #{Blufin::Terminal::format_directory(@dynamo_db_file)}", ['To download DynamoDB package, visit:', 'https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DynamoDBLocal.DownloadingAndRunning.html'], true) unless Blufin::Files::file_exists(@dynamo_db_file)

        end

        def opts_routing

            Blufin::Terminal::command("java -Djava.library.path=./DynamoDBLocal_lib -jar #{JAR_FILE} -sharedDb", @dynamo_db_path)

        end

    end

end